<?php
/*
Plugin Name: SQL Monitor
Description: SQL Query Debugger
Version: 0.5.4
Plugin URI: http://blog.sjinks.org.ua/wordpress-plugins/sqlmon/
Author: Vladimir Kolesnikov
Author URI: http://blog.sjinks.org.ua/
*/
    require_once(WP_PLUGIN_DIR . '/sqlmon/lib/class.Logger.php');
    require_once(WP_PLUGIN_DIR . '/sqlmon/lib/class.LogEntry.php');

    class SqlMonitor
    {
        /**
         * @var Logger
         */
        protected $logger;

        public static function instance()
        {
            static $self = false;
            if (false === $self) {
                $self = new SqlMonitor();
            }

            return $self;
        }

        protected function __construct()
        {
            global $wpdb;

            if ($wpdb instanceof DbProfile) {
                add_action('init', array(&$this, 'init'));
            }

            $this->logger = Logger::instance();

            add_action('activate_sqlmon/sqlmon.php',   array(&$this, 'activate'));
            add_action('deactivate_sqlmon/sqlmon.php', array(&$this, 'deactivate'));
        }

        public function init()
        {
            global $wpdb;

            $allow = apply_filters('enable_sqlmon', current_user_can('administrator'));
            if (true == $allow) {
                wp_enqueue_style('sqlmon-css', WP_PLUGIN_URL . '/sqlmon/sqldebug.css', array(), '', 'all');

                add_action('wp_footer',    array(&$this, 'footer'), 999999);
                add_action('admin_footer', array(&$this, 'footer'), 999999);
            }
            else {
                $wpdb->enabled = false;
            }
        }

        public function footer()
        {
            $this->logger->dump(false);
        }

        public function activate()
        {
            @copy(dirname(__FILE__) . '/db.php', WP_CONTENT_DIR . '/db.php');
            @chmod(WP_CONTENT_DIR . '/db.php', 0666);
        }

        public function deactivate()
        {
            if (false === @unlink(WP_CONTENT_DIR . '/db.php')) {
                trigger_error("Failed to delete " . WP_CONTENT_DIR . "/db.php. Please remove this file manually.", E_USER_WARNING);
            }
        }
    }

    /**
     * @desc Dumps the list of the queries
     * @param bool $clear Clear the query log
     * @param bool $force_css Forcefully include the CSS file
     */
    function sqlmon_dump_queries($clear = false, $force_css = false)
    {
        if (true == $force_css) {
            print "<link rel='stylesheet' type='text/css' href='" . WP_PLUGIN_URL . "/sqlmon/sqldebug.css'/>";
        }

        $logger = Logger::instance();
        $logger->dump();

        if (true == $clear) {
            $logger->clearLog();
        }
    }

    $SqlMonitor = SqlMonitor::instance();
?>